@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -312,7 +312,7 @@ ModuleExport unsigned long RegisterCALSImage(void)
   static const char
     *CALSDescription=
     {
-      "Continuous Acquisition and Life-cycle Support Type 1 Image"
+      "Continuous Acquisition and Life-cycle Support Type 1"
     },
     *CALSNote=
     {
@@ -369,6 +369,7 @@ ModuleExport void UnregisterCALSImage(void)
   (void) UnregisterMagickInfo("CALS");
 }
 
+#if defined(MAGICKCORE_TIFF_DELEGATE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -541,7 +542,7 @@ static MagickBooleanType WriteCALSImage(const ImageInfo *image_info,
         geometry_info;
 
       (void) ParseGeometry(image_info->density,&geometry_info);
-      density=(unsigned long) (geometry_info.rho+0.5);
+      density=(unsigned long) floor(geometry_info.rho+0.5);
     }
   (void) FormatMagickString(header,MaxTextExtent,"rdensty: %04lu",density);
   count=WriteCALSRecord(image,header);
@@ -576,3 +577,4 @@ static MagickBooleanType WriteCALSImage(const ImageInfo *image_info,
   (void) CloseBlob(image);
   return(status);
 }
+#endif
