@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -41,21 +41,32 @@
 */
 #include "magick/studio.h"
 #include "magick/annotate.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
-#include "magick/color_private.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
 #include "magick/draw.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/option.h"
+#include "magick/pixel-private.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
+#include "magick/statistic.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 
 /*
   Forward declarations.
@@ -83,34 +94,33 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsTXT(const unsigned char *magick,const size_t length)
 {
 #define MagickID  "# ImageMagick pixel enumeration:"
 
   char
-    format[MaxTextExtent];
+    colorspace[MaxTextExtent];
 
   ssize_t
     count;
 
   unsigned long
     columns,
-    max_value,
+    depth,
     rows;
 
-  if (length < 32)
+  if (length < 40)
     return(MagickFalse);
-  count=(ssize_t) sscanf((char *) magick+32,"%lu,%lu,%lu,%s",&columns,&rows,
-    &max_value,format);
-  if ((LocaleNCompare((char *) magick,MagickID,strlen(MagickID)) != 0) ||
-      (count != 4))
+  if (LocaleNCompare((const char *) magick,MagickID,strlen(MagickID)) != 0)
+    return(MagickFalse);
+  count=(ssize_t) sscanf((const char *) magick+32,"%lu,%lu,%lu,%s",&columns,
+    &rows,&depth,colorspace);
+  if (count != 4)
     return(MagickFalse);
   return(MagickTrue);
 }
@@ -132,35 +142,32 @@ static MagickBooleanType IsTXT(const unsigned char *magick,const size_t length)
 %
 %  The format of the ReadTEXTImage method is:
 %
-%      Image *ReadTEXTImage(const ImageInfo *image_info,
-%        ExceptionInfo *exception)
+%      Image *ReadTEXTImage(const ImageInfo *image_info,Image *image,
+%        char *text,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
+%
+%    o image: the image.
+%
+%    o text: the text storage buffer.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
-static Image *ReadTEXTImage(const ImageInfo *image_info,
-  ExceptionInfo *exception)
+static Image *ReadTEXTImage(const ImageInfo *image_info,Image *image,
+  char *text,ExceptionInfo *exception)
 {
   char
     filename[MaxTextExtent],
     geometry[MaxTextExtent],
-    *p,
-    text[MaxTextExtent];
-
-  double
-    dx_resolution,
-    dy_resolution;
+    *p;
 
   DrawInfo
     *draw_info;
 
   Image
-    *image,
     *texture;
 
   long
@@ -169,6 +176,9 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
+  PointInfo
+    delta;
+
   RectangleInfo
     page;
 
@@ -181,21 +191,15 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == MagickFalse)
-    {
-      DestroyImageList(image);
-      return((Image *) NULL);
-    }
   /*
     Set the page geometry.
   */
-  dx_resolution=72.0;
-  dy_resolution=72.0;
+  delta.x=DefaultResolution;
+  delta.y=DefaultResolution;
   if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
     {
       GeometryInfo
@@ -219,10 +223,12 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
   /*
     Initialize Image structure.
   */
-  image->columns=(unsigned long)
-    (((page.width*image->x_resolution)/dx_resolution)+0.5);
-  image->rows=(unsigned long)
-    (((page.height*image->y_resolution)/dy_resolution)+0.5);
+  image->columns=(unsigned long) floor(((page.width*image->x_resolution)/
+    delta.x)+0.5);
+  image->rows=(unsigned long) floor(((page.height*image->y_resolution)/
+    delta.y)+0.5);
+  image->page.x=0;
+  image->page.y=0;
   texture=(Image *) NULL;
   if (image_info->texture != (char *) NULL)
     {
@@ -230,56 +236,57 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
         *read_info;
 
       read_info=CloneImageInfo(image_info);
-      read_info->blob=(void *) NULL;
-      read_info->length=0;
+      SetImageInfoBlob(read_info,(void *) NULL,0);
       (void) CopyMagickString(read_info->filename,image_info->texture,
         MaxTextExtent);
       texture=ReadImage(read_info,exception);
-      DestroyImageInfo(read_info);
+      read_info=DestroyImageInfo(read_info);
     }
   /*
     Annotate the text image.
   */
-  SetImage(image,OpaqueOpacity);
+  (void) SetImageBackgroundColor(image);
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
-  draw_info->fill=image_info->pen;
   (void) CloneString(&draw_info->text,image_info->filename);
   (void) FormatMagickString(geometry,MaxTextExtent,"0x0%+ld%+ld",page.x,page.y);
   (void) CloneString(&draw_info->geometry,geometry);
   status=GetTypeMetrics(image,draw_info,&metrics);
   if (status == MagickFalse)
     ThrowReaderException(TypeError,"UnableToGetTypeMetrics");
+  page.y=(long) ceil(page.y+metrics.ascent-0.5);
+  (void) FormatMagickString(geometry,MaxTextExtent,"0x0%+ld%+ld",page.x,page.y);
+  (void) CloneString(&draw_info->geometry,geometry);
   (void) CopyMagickString(filename,image_info->filename,MaxTextExtent);
   if (*draw_info->text != '\0')
     *draw_info->text='\0';
-  p=ReadBlobString(image,text);
+  p=text;
   for (offset=2*page.y; p != (char *) NULL; )
   {
     /*
       Annotate image with text.
     */
     (void) ConcatenateString(&draw_info->text,text);
-    (void) ConcatenateString(&draw_info->text,"\\n");
+    (void) ConcatenateString(&draw_info->text,"\n");
     offset+=(long) (metrics.ascent-metrics.descent);
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(offset,image->rows))
-        {
-          status=MagickMonitor(LoadImageTag,offset,image->rows,
-            &image->exception);
-          if (status == MagickFalse)
-            break;
-        }
+      {
+        status=SetImageProgress(image,LoadImageTag,offset,image->rows);
+        if (status == MagickFalse)
+          break;
+      }
     p=ReadBlobString(image,text);
     if ((offset < (long) image->rows) && (p != (char *) NULL))
       continue;
     if (texture != (Image *) NULL)
       {
-        MonitorHandler
-          handler;
+        MagickProgressMonitor
+          progress_monitor;
 
-        handler=SetMonitorHandler((MonitorHandler) NULL);
-        TextureImage(image,texture);
-        (void) SetMonitorHandler(handler);
+        progress_monitor=SetImageProgressMonitor(image,
+          (MagickProgressMonitor) NULL,image->client_data);
+        (void) TextureImage(image,texture);
+        (void) SetImageProgressMonitor(image,progress_monitor,
+          image->client_data);
       }
     (void) AnnotateImage(image,draw_info);
     if (p == (char *) NULL)
@@ -289,36 +296,37 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
     */
     *draw_info->text='\0';
     offset=2*page.y;
-    AllocateNextImage(image_info,image);
-    if (image->next == (Image *) NULL)
+    AcquireNextImage(image_info,image);
+    if (GetNextImageInList(image) == (Image *) NULL)
       {
-        DestroyImageList(image);
+        image=DestroyImageList(image);
         return((Image *) NULL);
       }
     image->next->columns=image->columns;
     image->next->rows=image->rows;
     image=SyncNextImageInList(image);
     (void) CopyMagickString(image->filename,filename,MaxTextExtent);
-    SetImage(image,OpaqueOpacity);
-    status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-      exception);
+    (void) SetImageBackgroundColor(image);
+    status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+      GetBlobSize(image));
     if (status == MagickFalse)
       break;
   }
   if (texture != (Image *) NULL)
     {
-      MonitorHandler
-        handler;
+      MagickProgressMonitor
+        progress_monitor;
 
-      handler=SetMonitorHandler((MonitorHandler) NULL);
+      progress_monitor=SetImageProgressMonitor(image,
+        (MagickProgressMonitor) NULL,image->client_data);
       (void) TextureImage(image,texture);
-      (void) SetMonitorHandler(handler);
+      (void) SetImageProgressMonitor(image,progress_monitor,image->client_data);
     }
   (void) AnnotateImage(image,draw_info);
   if (texture != (Image *) NULL)
-    DestroyImage(texture);
-  DestroyDrawInfo(draw_info);
-  CloseBlob(image);
+    texture=DestroyImage(texture);
+  draw_info=DestroyDrawInfo(draw_info);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -343,22 +351,25 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   char
-    format[MaxTextExtent],
+    colorspace[MaxTextExtent],
     text[MaxTextExtent];
 
   Image
     *image;
 
+  IndexPacket
+    *indexes;
+
   long
+    type,
     x,
     y;
 
@@ -368,8 +379,8 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  Quantum
-    *scale;
+  QuantumAny
+    range;
 
   register long
     i;
@@ -381,6 +392,7 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     count;
 
   unsigned long
+    depth,
     max_value;
 
   /*
@@ -389,73 +401,80 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  (void) ResetMagickMemory(text,0,sizeof(text));
   (void) ReadBlobString(image,text);
-  count=(ssize_t) sscanf(text+32,"%lu,%lu,%lu,%s",&image->columns,&image->rows,
-    &max_value,format);
-  if ((LocaleNCompare(text,"# ImageMagick pixel enumeration:",32) != 0) ||
-      (count != 4))
+  if (LocaleNCompare((char *) text,MagickID,strlen(MagickID)) != 0)
+    return(ReadTEXTImage(image_info,image,text,exception));
+  *colorspace='\0';
+  count=(ssize_t) sscanf(text+32,"%lu,%lu,%lu,%s",&image->columns,
+    &image->rows,&max_value,colorspace);
+  if (count != 4)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  if (LocaleNCompare(format,"CMYK",4) == 0)
-    image->colorspace=CMYKColorspace;
-  image->matte=(MagickBooleanType) (strchr(format,'A') != (char *) NULL);
-  scale=(Quantum *) NULL;
-  if (max_value != MaxRGB)
+  for (depth=1; (GetQuantumRange(depth)+1) < max_value; depth++) ;
+  image->depth=depth;
+  LocaleLower(colorspace);
+  i=(long) strlen(colorspace)-1;
+  image->matte=MagickFalse;
+  if ((i > 0) && (colorspace[i] == 'a'))
     {
-      /*
-        Compute pixel scaling table.
-      */
-      scale=(Quantum *)
-        AcquireMagickMemory((size_t) (max_value+1)*sizeof(*scale));
-      if (scale == (Quantum *) NULL)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      for (i=0; i <= (long) max_value; i++)
-        scale[i]=ScaleXToQuantum(i,max_value);
+      colorspace[i]='\0';
+      image->matte=MagickTrue;
     }
-  pixel.opacity=OpaqueOpacity;
+  type=ParseMagickOption(MagickColorspaceOptions,MagickFalse,colorspace);
+  if (type < 0)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  image->colorspace=(ColorspaceType) type;
+  (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
+  (void) SetImageBackgroundColor(image);
+  range=GetQuantumRange(image->depth);
   while (ReadBlobString(image,text) != (char *) NULL)
   {
-    if (image->matte == MagickFalse)
+    if (image->colorspace == CMYKColorspace)
       {
-        count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu)",&x,&y,&pixel.red,
-          &pixel.green,&pixel.blue);
-        if (count != 5)
-          continue;
+        if (image->matte != MagickFalse)
+          count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu,%lu,%lu",&x,&y,
+            &pixel.red,&pixel.green,&pixel.blue,&pixel.index,&pixel.opacity);
+        else
+          count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu,%lu",&x,&y,
+            &pixel.red,&pixel.green,&pixel.blue,&pixel.index);
       }
     else
-      {
-        count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu,%lu)",&x,&y,
+      if (image->matte != MagickFalse)
+        count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu,%lu",&x,&y,
           &pixel.red,&pixel.green,&pixel.blue,&pixel.opacity);
-        if (count != 6)
-          continue;
-      }
-    if (scale != (Quantum *) NULL)
-      {
-        pixel.red=scale[pixel.red];
-        pixel.green=scale[pixel.green];
-        pixel.blue=scale[pixel.blue];
-        pixel.opacity=scale[pixel.opacity];
-      }
-    q=SetImagePixels(image,x,y,1,1);
+      else
+        count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu",&x,&y,
+          &pixel.red,&pixel.green,&pixel.blue);
+    if (count < 5)
+      continue;
+    q=GetAuthenticPixels(image,x,y,1,1,exception);
     if (q == (PixelPacket *) NULL)
       continue;
-    q->red=(Quantum) pixel.red;
-    q->green=(Quantum) pixel.green;
-    q->blue=(Quantum) pixel.blue;
-    q->opacity=(Quantum) pixel.opacity;
-    (void) SyncImagePixels(image);
+    q->red=ScaleAnyToQuantum(pixel.red,range);
+    q->green=ScaleAnyToQuantum(pixel.green,range);
+    q->blue=ScaleAnyToQuantum(pixel.blue,range);
+    if (image->colorspace == CMYKColorspace)
+      {
+        indexes=GetAuthenticIndexQueue(image);
+        *indexes=ScaleAnyToQuantum(pixel.index,range);
+      }
+    if (image->matte != MagickFalse)
+      q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel.opacity,
+        range));
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
+      break;
   }
-  if (scale != (Quantum *) NULL)
-    scale=(Quantum *) RelinquishMagickMemory(scale);
   return(GetFirstImageInList(image));
 }
 
@@ -470,8 +489,8 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RegisterTXTImage() adds attributes for the TXT image format to
-%  the list of supported formats.  The attributes include the image format
+%  RegisterTXTImage() adds attributes for the TXT image format to the
+%  list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
 %  whether the format supports native in-memory I/O, and a brief
@@ -479,29 +498,32 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterTXTImage method is:
 %
-%      RegisterTXTImage(void)
+%      unsigned long RegisterTXTImage(void)
 %
 */
-ModuleExport void RegisterTXTImage(void)
+ModuleExport unsigned long RegisterTXTImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("TEXT");
-  entry->decoder=(DecoderHandler *) ReadTEXTImage;
-  entry->encoder=(EncoderHandler *) WriteTXTImage;
+  entry->decoder=(DecodeImageHandler *) ReadTXTImage;
+  entry->encoder=(EncodeImageHandler *) WriteTXTImage;
   entry->raw=MagickTrue;
+  entry->adjoin=MagickFalse;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Text");
-  entry->module=AcquireString("TXT");
+  entry->description=ConstantString("Text");
+  entry->module=ConstantString("TXT");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TXT");
-  entry->decoder=(DecoderHandler *) ReadTXTImage;
-  entry->encoder=(EncoderHandler *) WriteTXTImage;
-  entry->description=AcquireString("Text");
-  entry->magick=(MagickHandler *) IsTXT;
-  entry->module=AcquireString("TXT");
+  entry->decoder=(DecodeImageHandler *) ReadTXTImage;
+  entry->encoder=(EncodeImageHandler *) WriteTXTImage;
+  entry->adjoin=MagickFalse;
+  entry->description=ConstantString("Text");
+  entry->magick=(IsImageFormatHandler *) IsTXT;
+  entry->module=ConstantString("TXT");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -516,7 +538,7 @@ ModuleExport void RegisterTXTImage(void)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  UnregisterTXTImage() removes format registrations made by the
-%  TXT module from the list of supported formats.
+%  TXT module from the list of supported format.
 %
 %  The format of the UnregisterTXTImage method is:
 %
@@ -548,17 +570,16 @@ ModuleExport void UnregisterTXTImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
 {
   char
     buffer[MaxTextExtent],
-    format[MaxTextExtent],
+    colorspace[MaxTextExtent],
     tuple[MaxTextExtent];
 
   long
@@ -567,12 +588,12 @@ static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
-  MagickOffsetType
-    scene;
-
   MagickPixelPacket
     pixel;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
@@ -587,66 +608,65 @@ static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  if (image->colorspace == CMYKColorspace)
-    (void) strcpy(format,"CMYK");
-  else
-    (void) strcpy(format,"RGB");
+  (void) CopyMagickString(colorspace,MagickOptionToMnemonic(
+    MagickColorspaceOptions,(long) image->colorspace),MaxTextExtent);
+  LocaleLower(colorspace);
+  image->depth=GetImageQuantumDepth(image,MagickTrue);
   if (image->matte != MagickFalse)
-    (void) ConcatenateMagickString(format,"A",MaxTextExtent);
+    (void) ConcatenateMagickString(colorspace,"a",MaxTextExtent);
   (void) FormatMagickString(buffer,MaxTextExtent,
     "# ImageMagick pixel enumeration: %lu,%lu,%lu,%s\n",image->columns,
-    image->rows,image->depth == QuantumDepth ? (unsigned long) MaxRGB :
-    (1L << image->depth)-1,format);
+    image->rows,(unsigned long) GetQuantumRange(image->depth),colorspace);
   (void) WriteBlobString(image,buffer);
-  scene=0;
-  do
+  GetMagickPixelPacket(image,&pixel);
+  for (y=0; y < (long) image->rows; y++)
   {
-    /*
-      Convert MIFF to TXT raster pixels.
-    */
-    (void) SetImageColorspace(image,RGBColorspace);
-    GetMagickPixelPacket(image,&pixel);
-    for (y=0; y < (long) image->rows; y++)
-    {
-      p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-      if (p == (const PixelPacket *) NULL)
-        break;
-      for (x=0; x < (long) image->columns; x++)
-      {
-        (void) FormatMagickString(buffer,MaxTextExtent,"%ld,%ld: ",x,y);
-        (void) WriteBlobString(image,buffer);
-        pixel.red=p->red;
-        pixel.green=p->green;
-        pixel.blue=p->blue;
-        pixel.opacity=p->opacity;
-        GetColorTuple(&pixel,MagickFalse,tuple);
-        (void) ConcatenateMagickString(tuple," ",MaxTextExtent);
-        (void) WriteBlobString(image,tuple);
-        (void) QueryColorname(image,p,SVGCompliance,tuple,&image->exception);
-        (void) WriteBlobString(image,tuple);
-        (void) WriteBlobString(image,"\n");
-        p++;
-      }
-      if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != MagickFalse)
-          {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
-            if (status == MagickFalse)
-              break;
-          }
-    }
-    if (image->next == (Image *) NULL)
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
+    if (p == (const PixelPacket *) NULL)
       break;
-    image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
+    indexes=GetVirtualIndexQueue(image);
+    for (x=0; x < (long) image->columns; x++)
+    {
+      (void) FormatMagickString(buffer,MaxTextExtent,"%ld,%ld: ",x,y);
+      (void) WriteBlobString(image,buffer);
+      SetMagickPixelPacket(image,p,indexes+x,&pixel);
+      (void) CopyMagickString(tuple,"(",MaxTextExtent);
+      ConcatenateColorComponent(&pixel,RedChannel,X11Compliance,tuple);
+      (void) ConcatenateMagickString(tuple,",",MaxTextExtent);
+      ConcatenateColorComponent(&pixel,GreenChannel,X11Compliance,tuple);
+      (void) ConcatenateMagickString(tuple,",",MaxTextExtent);
+      ConcatenateColorComponent(&pixel,BlueChannel,X11Compliance,tuple);
+      if (pixel.colorspace == CMYKColorspace)
+        {
+          (void) ConcatenateMagickString(tuple,",",MaxTextExtent);
+          ConcatenateColorComponent(&pixel,IndexChannel,X11Compliance,tuple);
+        }
+      if (pixel.matte != MagickFalse)
+        {
+          (void) ConcatenateMagickString(tuple,",",MaxTextExtent);
+          ConcatenateColorComponent(&pixel,AlphaChannel,X11Compliance,tuple);
+        }
+      (void) ConcatenateMagickString(tuple,")",MaxTextExtent);
+      (void) WriteBlobString(image,tuple);
+      (void) WriteBlobString(image,"  ");
+      GetColorTuple(&pixel,MagickTrue,tuple);
+      (void) FormatMagickString(buffer,MaxTextExtent,"%s",tuple);
+      (void) WriteBlobString(image,buffer);
+      (void) WriteBlobString(image,"  ");
+      (void) QueryMagickColorname(image,&pixel,SVGCompliance,tuple,
+        &image->exception);
+      (void) WriteBlobString(image,tuple);
+      (void) WriteBlobString(image,"\n");
+      p++;
+    }
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
     if (status == MagickFalse)
       break;
-  } while (image_info->adjoin != MagickFalse);
-  CloseBlob(image);
+  }
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
