@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -503,7 +503,7 @@ MagickExport long FormatMagickCaption(Image *image,DrawInfo *draw_info,
     status=GetTypeMetrics(image,draw_info,metrics);
     if (status == MagickFalse)
       break;
-    width=(unsigned long) (metrics->width+0.5);
+    width=(unsigned long) floor(metrics->width+0.5);
     if (GetUTFCode(p) != '\n')
       if (width <= image->columns)
         continue;
@@ -817,14 +817,19 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
           TypeWarning,"UnableToReadFont","`%s'",draw_info->family);
     }
   if (type_info == (const TypeInfo *) NULL)
-    type_info=GetTypeInfoByFamily("arial",draw_info->style,
+    type_info=GetTypeInfoByFamily("Arial",draw_info->style,
       draw_info->stretch,draw_info->weight,&image->exception);
   if (type_info == (const TypeInfo *) NULL)
-    type_info=GetTypeInfoByFamily("helvetica",draw_info->style,
+    type_info=GetTypeInfoByFamily("Helvetica",draw_info->style,
+      draw_info->stretch,draw_info->weight,&image->exception);
+  if (type_info == (const TypeInfo *) NULL)
+    type_info=GetTypeInfoByFamily("Century Schoolbook",draw_info->style,
       draw_info->stretch,draw_info->weight,&image->exception);
   if (type_info == (const TypeInfo *) NULL)
     type_info=GetTypeInfoByFamily((const char *) NULL,draw_info->style,
       draw_info->stretch,draw_info->weight,&image->exception);
+  if (type_info == (const TypeInfo *) NULL)
+    type_info=GetTypeInfo("*",&image->exception);
   if (type_info == (const TypeInfo *) NULL)
     {
       status=RenderFreetype(image,draw_info,draw_info->encoding,offset,metrics);
@@ -885,9 +890,10 @@ static int TraceCubicBezier(FT_Vector *p,FT_Vector *q,FT_Vector *to,
     path[MaxTextExtent];
 
   affine=draw_info->affine;
-  (void) FormatMagickString(path,MaxTextExtent,"C%g,%g %g,%g %g,%g",
-    affine.tx+p->x/64.0,affine.ty-p->y/64.0,affine.tx+q->x/64.0,
-    affine.ty-q->y/64.0,affine.tx+to->x/64.0,affine.ty-to->y/64.0);
+  (void) FormatMagickString(path,MaxTextExtent,
+    "C%g,%g %g,%g %g,%g",affine.tx+p->x/64.0,affine.ty-
+    p->y/64.0,affine.tx+q->x/64.0,affine.ty-q->y/64.0,affine.tx+to->x/64.0,
+    affine.ty-to->y/64.0);
   (void) ConcatenateString(&draw_info->primitive,path);
   return(0);
 }
@@ -901,8 +907,8 @@ static int TraceLineTo(FT_Vector *to,DrawInfo *draw_info)
     path[MaxTextExtent];
 
   affine=draw_info->affine;
-  (void) FormatMagickString(path,MaxTextExtent,"L%g,%g",affine.tx+to->x/64.0,
-    affine.ty-to->y/64.0);
+  (void) FormatMagickString(path,MaxTextExtent,"L%g,%g",affine.tx+
+    to->x/64.0,affine.ty-to->y/64.0);
   (void) ConcatenateString(&draw_info->primitive,path);
   return(0);
 }
@@ -916,8 +922,8 @@ static int TraceMoveTo(FT_Vector *to,DrawInfo *draw_info)
     path[MaxTextExtent];
 
   affine=draw_info->affine;
-  (void) FormatMagickString(path,MaxTextExtent,"M%g,%g",affine.tx+to->x/64.0,
-    affine.ty-to->y/64.0);
+  (void) FormatMagickString(path,MaxTextExtent,"M%g,%g",affine.tx+
+    to->x/64.0,affine.ty-to->y/64.0);
   (void) ConcatenateString(&draw_info->primitive,path);
   return(0);
 }
@@ -1176,180 +1182,366 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   point.y=0.0;
   code=0;
   for (p=draw_info->text; GetUTFCode(p) != 0; p+=GetUTFOctets(p))
-  {
-    glyph.id=FT_Get_Char_Index(face,GetUTFCode(p));
-    if (glyph.id == 0)
-      glyph.id=FT_Get_Char_Index(face,'?');
-    if ((glyph.id != 0) && (last_glyph.id != 0))
-      {
-        if (draw_info->kerning != 0.0)
-          origin.x+=64.0*draw_info->kerning;
-        else
-          if (FT_HAS_KERNING(face))
-            {
-              FT_Vector
-                kerning;
-
-              status=FT_Get_Kerning(face,last_glyph.id,glyph.id,
-                ft_kerning_default,&kerning);
-              if (status == 0)
-                origin.x+=kerning.x;
-            }
-        }
-    glyph.origin=origin;
-    status=FT_Load_Glyph(face,glyph.id,flags);
-    if (status != 0)
-      continue;
-    status=FT_Get_Glyph(face->glyph,&glyph.image);
-    if (status != 0)
-      continue;
-    status=FT_Outline_Get_BBox(&((FT_OutlineGlyph) glyph.image)->outline,
-      &bounds);
-    if (status != 0)
-      continue;
-    if ((p == draw_info->text) || (bounds.xMin < metrics->bounds.x1))
-      metrics->bounds.x1=bounds.xMin;
-    if ((p == draw_info->text) || (bounds.yMin < metrics->bounds.y1))
-      metrics->bounds.y1=bounds.yMin;
-    if ((p == draw_info->text) || (bounds.xMax > metrics->bounds.x2))
-      metrics->bounds.x2=bounds.xMax;
-    if ((p == draw_info->text) || (bounds.yMax > metrics->bounds.y2))
-      metrics->bounds.y2=bounds.yMax;
-    if (draw_info->render != MagickFalse)
-      if ((draw_info->stroke.opacity != TransparentOpacity) ||
-          (draw_info->stroke_pattern != (Image *) NULL))
+    if (GetUTFCode(p) < 0)
+      break;
+  if (GetUTFCode(p) == 0)
+    for (p=draw_info->text; GetUTFCode(p) != 0; p+=GetUTFOctets(p))
+    {
+      /*
+        Render UTF-8 sequence.
+      */
+      glyph.id=FT_Get_Char_Index(face,GetUTFCode(p));
+      if (glyph.id == 0)
+        glyph.id=FT_Get_Char_Index(face,'?');
+      if ((glyph.id != 0) && (last_glyph.id != 0))
         {
-          /*
-            Trace the glyph.
-          */
-          annotate_info->affine.tx=glyph.origin.x/64.0;
-          annotate_info->affine.ty=glyph.origin.y/64.0;
-          (void) FT_Outline_Decompose(&((FT_OutlineGlyph) glyph.image)->outline,
-            &OutlineMethods,annotate_info);
-        }
-    FT_Vector_Transform(&glyph.origin,&affine);
-    (void) FT_Glyph_Transform(glyph.image,&affine,&glyph.origin);
-    status=FT_Glyph_To_Bitmap(&glyph.image,ft_render_mode_normal,
-      (FT_Vector *) NULL,MagickTrue);
-    if (status != 0)
-      continue;
-    bitmap=(FT_BitmapGlyph) glyph.image;
-    point.x=offset->x+bitmap->left;
-    point.y=offset->y-bitmap->top;
-    if (draw_info->render != MagickFalse)
-      {
-        CacheView
-          *image_view;
+          if (draw_info->kerning != 0.0)
+            origin.x+=64.0*draw_info->kerning;
+          else
+            if (FT_HAS_KERNING(face))
+              {
+                FT_Vector
+                  kerning;
 
-        ExceptionInfo
-          *exception;
-
-        MagickBooleanType
-          status;
-
-        /*
-          Rasterize the glyph.
-        */
-        status=MagickTrue;
-        exception=(&image->exception);
-        image_view=AcquireCacheView(image);
-        for (y=0; y < (long) bitmap->bitmap.rows; y++)
+                status=FT_Get_Kerning(face,last_glyph.id,glyph.id,
+                  ft_kerning_default,&kerning);
+                if (status == 0)
+                  origin.x+=kerning.x;
+              }
+          }
+      glyph.origin=origin;
+      status=FT_Load_Glyph(face,glyph.id,flags);
+      if (status != 0)
+        continue;
+      status=FT_Get_Glyph(face->glyph,&glyph.image);
+      if (status != 0)
+        continue;
+      status=FT_Outline_Get_BBox(&((FT_OutlineGlyph) glyph.image)->outline,
+        &bounds);
+      if (status != 0)
+        continue;
+      if ((p == draw_info->text) || (bounds.xMin < metrics->bounds.x1))
+        metrics->bounds.x1=bounds.xMin;
+      if ((p == draw_info->text) || (bounds.yMin < metrics->bounds.y1))
+        metrics->bounds.y1=bounds.yMin;
+      if ((p == draw_info->text) || (bounds.xMax > metrics->bounds.x2))
+        metrics->bounds.x2=bounds.xMax;
+      if ((p == draw_info->text) || (bounds.yMax > metrics->bounds.y2))
+        metrics->bounds.y2=bounds.yMax;
+      if (draw_info->render != MagickFalse)
+        if ((draw_info->stroke.opacity != TransparentOpacity) ||
+            (draw_info->stroke_pattern != (Image *) NULL))
+          {
+            /*
+              Trace the glyph.
+            */
+            annotate_info->affine.tx=glyph.origin.x/64.0;
+            annotate_info->affine.ty=glyph.origin.y/64.0;
+            (void) FT_Outline_Decompose(&((FT_OutlineGlyph) glyph.image)->
+              outline,&OutlineMethods,annotate_info);
+          }
+      FT_Vector_Transform(&glyph.origin,&affine);
+      (void) FT_Glyph_Transform(glyph.image,&affine,&glyph.origin);
+      status=FT_Glyph_To_Bitmap(&glyph.image,ft_render_mode_normal,
+        (FT_Vector *) NULL,MagickTrue);
+      if (status != 0)
+        continue;
+      bitmap=(FT_BitmapGlyph) glyph.image;
+      point.x=offset->x+bitmap->left;
+      point.y=offset->y-bitmap->top;
+      if (draw_info->render != MagickFalse)
         {
-          long
-            x_offset,
-            y_offset;
+          CacheView
+            *image_view;
+
+          ExceptionInfo
+            *exception;
 
           MagickBooleanType
-            active,
-            sync;
+            status;
 
-          MagickRealType
-            fill_opacity;
-
-          PixelPacket
-            fill_color;
-
-          register long
-            x;
-
-          register PixelPacket
-            *__restrict q;
-
-          register unsigned char
-            *p;
-
-          if (status == MagickFalse)
-            continue;
-          x_offset=(long) (point.x+0.5);
-          y_offset=(long) (point.y+y+0.5);
-          if ((y_offset < 0) || (y_offset >= (long) image->rows))
-            continue;
-          q=(PixelPacket *) NULL;
-          if ((x_offset < 0) || (x_offset >= (long) image->columns))
-            active=MagickFalse;
-          else
-            {
-              q=GetCacheViewAuthenticPixels(image_view,x_offset,y_offset,
-                bitmap->bitmap.width,1,exception);
-              active=q != (PixelPacket *) NULL ? MagickTrue : MagickFalse;
-            }
-          p=bitmap->bitmap.buffer+y*bitmap->bitmap.width;
-          for (x=0; x < (long) bitmap->bitmap.width; x++)
+          /*
+            Rasterize the glyph.
+          */
+          status=MagickTrue;
+          exception=(&image->exception);
+          image_view=AcquireCacheView(image);
+          for (y=0; y < (long) bitmap->bitmap.rows; y++)
           {
-            x_offset++;
-            if ((*p == 0) || (x_offset < 0) ||
-                (x_offset >= (long) image->columns))
+            long
+              x_offset,
+              y_offset;
+
+            MagickBooleanType
+              active,
+              sync;
+
+            MagickRealType
+              fill_opacity;
+
+            PixelPacket
+              fill_color;
+
+            register long
+              x;
+
+            register PixelPacket
+              *restrict q;
+
+            register unsigned char
+              *p;
+
+            if (status == MagickFalse)
+              continue;
+            x_offset=(long) ceil(point.x-0.5);
+            y_offset=(long) ceil(point.y+y-0.5);
+            if ((y_offset < 0) || (y_offset >= (long) image->rows))
+              continue;
+            q=(PixelPacket *) NULL;
+            if ((x_offset < 0) || (x_offset >= (long) image->columns))
+              active=MagickFalse;
+            else
               {
-                p++;
-                q++;
-                continue;
+                q=GetCacheViewAuthenticPixels(image_view,x_offset,y_offset,
+                  bitmap->bitmap.width,1,exception);
+                active=q != (PixelPacket *) NULL ? MagickTrue : MagickFalse;
               }
-            fill_opacity=(MagickRealType) (*p)/(bitmap->bitmap.num_grays-1);
-            if (draw_info->text_antialias == MagickFalse)
-              fill_opacity=fill_opacity >= 0.5 ? 1.0 : 0.0;
-            if (active == MagickFalse)
-              q=GetCacheViewAuthenticPixels(image_view,x_offset,y_offset,1,1,
-                exception);
-            if (q == (PixelPacket *) NULL)
-              {
-                p++;
-                q++;
-                continue;
-              }
-            (void) GetFillColor(draw_info,x_offset,y_offset,&fill_color);
-            fill_opacity=QuantumRange-fill_opacity*(QuantumRange-
-              fill_color.opacity);
-            MagickCompositeOver(&fill_color,fill_opacity,q,q->opacity,q);
-            if (active == MagickFalse)
-              {
-                sync=SyncCacheViewAuthenticPixels(image_view,exception);
-                if (sync == MagickFalse)
-                  status=MagickFalse;
-              }
-            p++;
-            q++;
+            p=bitmap->bitmap.buffer+y*bitmap->bitmap.width;
+            for (x=0; x < (long) bitmap->bitmap.width; x++)
+            {
+              x_offset++;
+              if ((*p == 0) || (x_offset < 0) ||
+                  (x_offset >= (long) image->columns))
+                {
+                  p++;
+                  q++;
+                  continue;
+                }
+              fill_opacity=(MagickRealType) (*p)/(bitmap->bitmap.num_grays-1);
+              if (draw_info->text_antialias == MagickFalse)
+                fill_opacity=fill_opacity >= 0.5 ? 1.0 : 0.0;
+              if (active == MagickFalse)
+                q=GetCacheViewAuthenticPixels(image_view,x_offset,y_offset,1,1,
+                  exception);
+              if (q == (PixelPacket *) NULL)
+                {
+                  p++;
+                  q++;
+                  continue;
+                }
+              (void) GetFillColor(draw_info,x_offset,y_offset,&fill_color);
+              fill_opacity=QuantumRange-fill_opacity*(QuantumRange-
+                fill_color.opacity);
+              MagickCompositeOver(&fill_color,fill_opacity,q,q->opacity,q);
+              if (active == MagickFalse)
+                {
+                  sync=SyncCacheViewAuthenticPixels(image_view,exception);
+                  if (sync == MagickFalse)
+                    status=MagickFalse;
+                }
+              p++;
+              q++;
+            }
+            sync=SyncCacheViewAuthenticPixels(image_view,exception);
+            if (sync == MagickFalse)
+              status=MagickFalse;
           }
-          sync=SyncCacheViewAuthenticPixels(image_view,exception);
-          if (sync == MagickFalse)
-            status=MagickFalse;
+          image_view=DestroyCacheView(image_view);
         }
-        image_view=DestroyCacheView(image_view);
-      }
-    if ((bitmap->left+bitmap->bitmap.width) > metrics->width)
-      metrics->width=bitmap->left+bitmap->bitmap.width;
-    if ((draw_info->interword_spacing != 0.0) &&
-        (IsUTFSpace(GetUTFCode(p)) != MagickFalse) &&
-        (IsUTFSpace(code) == MagickFalse))
-      origin.x+=64.0*draw_info->interword_spacing;
-    else
-      origin.x+=face->glyph->advance.x;
-    metrics->origin.x=origin.x;
-    metrics->origin.y=origin.y;
-    if (last_glyph.id != 0)
-      FT_Done_Glyph(last_glyph.image);
-    last_glyph=glyph;
-    code=GetUTFCode(p);
-  }
+      if ((bitmap->left+bitmap->bitmap.width) > metrics->width)
+        metrics->width=bitmap->left+bitmap->bitmap.width;
+      if ((draw_info->interword_spacing != 0.0) &&
+          (IsUTFSpace(GetUTFCode(p)) != MagickFalse) &&
+          (IsUTFSpace(code) == MagickFalse))
+        origin.x+=64.0*draw_info->interword_spacing;
+      else
+        origin.x+=face->glyph->advance.x;
+      metrics->origin.x=origin.x;
+      metrics->origin.y=origin.y;
+      if (last_glyph.id != 0)
+        FT_Done_Glyph(last_glyph.image);
+      last_glyph=glyph;
+      code=GetUTFCode(p);
+    }
+  else
+    for (p=draw_info->text; *p != 0; p++)
+    {
+      /*
+        Render Latin-1 sequence.
+      */
+      glyph.id=FT_Get_Char_Index(face,(unsigned char) *p);
+      if (glyph.id == 0)
+        glyph.id=FT_Get_Char_Index(face,'?');
+      if ((glyph.id != 0) && (last_glyph.id != 0))
+        {
+          if (draw_info->kerning != 0.0)
+            origin.x+=64.0*draw_info->kerning;
+          else
+            if (FT_HAS_KERNING(face))
+              {
+                FT_Vector
+                  kerning;
+
+                status=FT_Get_Kerning(face,last_glyph.id,glyph.id,
+                  ft_kerning_default,&kerning);
+                if (status == 0)
+                  origin.x+=kerning.x;
+              }
+          }
+      glyph.origin=origin;
+      status=FT_Load_Glyph(face,glyph.id,flags);
+      if (status != 0)
+        continue;
+      status=FT_Get_Glyph(face->glyph,&glyph.image);
+      if (status != 0)
+        continue;
+      status=FT_Outline_Get_BBox(&((FT_OutlineGlyph) glyph.image)->outline,
+        &bounds);
+      if (status != 0)
+        continue;
+      if ((p == draw_info->text) || (bounds.xMin < metrics->bounds.x1))
+        metrics->bounds.x1=bounds.xMin;
+      if ((p == draw_info->text) || (bounds.yMin < metrics->bounds.y1))
+        metrics->bounds.y1=bounds.yMin;
+      if ((p == draw_info->text) || (bounds.xMax > metrics->bounds.x2))
+        metrics->bounds.x2=bounds.xMax;
+      if ((p == draw_info->text) || (bounds.yMax > metrics->bounds.y2))
+        metrics->bounds.y2=bounds.yMax;
+      if (draw_info->render != MagickFalse)
+        if ((draw_info->stroke.opacity != TransparentOpacity) ||
+            (draw_info->stroke_pattern != (Image *) NULL))
+          {
+            /*
+              Trace the glyph.
+            */
+            annotate_info->affine.tx=glyph.origin.x/64.0;
+            annotate_info->affine.ty=glyph.origin.y/64.0;
+            (void) FT_Outline_Decompose(&((FT_OutlineGlyph) glyph.image)->
+              outline,&OutlineMethods,annotate_info);
+          }
+      FT_Vector_Transform(&glyph.origin,&affine);
+      (void) FT_Glyph_Transform(glyph.image,&affine,&glyph.origin);
+      status=FT_Glyph_To_Bitmap(&glyph.image,ft_render_mode_normal,
+        (FT_Vector *) NULL,MagickTrue);
+      if (status != 0)
+        continue;
+      bitmap=(FT_BitmapGlyph) glyph.image;
+      point.x=offset->x+bitmap->left;
+      point.y=offset->y-bitmap->top;
+      if (draw_info->render != MagickFalse)
+        {
+          CacheView
+            *image_view;
+
+          ExceptionInfo
+            *exception;
+
+          MagickBooleanType
+            status;
+
+          /*
+            Rasterize the glyph.
+          */
+          status=MagickTrue;
+          exception=(&image->exception);
+          image_view=AcquireCacheView(image);
+          for (y=0; y < (long) bitmap->bitmap.rows; y++)
+          {
+            long
+              x_offset,
+              y_offset;
+
+            MagickBooleanType
+              active,
+              sync;
+
+            MagickRealType
+              fill_opacity;
+
+            PixelPacket
+              fill_color;
+
+            register long
+              x;
+
+            register PixelPacket
+              *restrict q;
+
+            register unsigned char
+              *p;
+
+            if (status == MagickFalse)
+              continue;
+            x_offset=(long) ceil(point.x-0.5);
+            y_offset=(long) ceil(point.y+y-0.5);
+            if ((y_offset < 0) || (y_offset >= (long) image->rows))
+              continue;
+            q=(PixelPacket *) NULL;
+            if ((x_offset < 0) || (x_offset >= (long) image->columns))
+              active=MagickFalse;
+            else
+              {
+                q=GetCacheViewAuthenticPixels(image_view,x_offset,y_offset,
+                  bitmap->bitmap.width,1,exception);
+                active=q != (PixelPacket *) NULL ? MagickTrue : MagickFalse;
+              }
+            p=bitmap->bitmap.buffer+y*bitmap->bitmap.width;
+            for (x=0; x < (long) bitmap->bitmap.width; x++)
+            {
+              x_offset++;
+              if ((*p == 0) || (x_offset < 0) ||
+                  (x_offset >= (long) image->columns))
+                {
+                  p++;
+                  q++;
+                  continue;
+                }
+              fill_opacity=(MagickRealType) (*p)/(bitmap->bitmap.num_grays-1);
+              if (draw_info->text_antialias == MagickFalse)
+                fill_opacity=fill_opacity >= 0.5 ? 1.0 : 0.0;
+              if (active == MagickFalse)
+                q=GetCacheViewAuthenticPixels(image_view,x_offset,y_offset,1,1,
+                  exception);
+              if (q == (PixelPacket *) NULL)
+                {
+                  p++;
+                  q++;
+                  continue;
+                }
+              (void) GetFillColor(draw_info,x_offset,y_offset,&fill_color);
+              fill_opacity=QuantumRange-fill_opacity*(QuantumRange-
+                fill_color.opacity);
+              MagickCompositeOver(&fill_color,fill_opacity,q,q->opacity,q);
+              if (active == MagickFalse)
+                {
+                  sync=SyncCacheViewAuthenticPixels(image_view,exception);
+                  if (sync == MagickFalse)
+                    status=MagickFalse;
+                }
+              p++;
+              q++;
+            }
+            sync=SyncCacheViewAuthenticPixels(image_view,exception);
+            if (sync == MagickFalse)
+              status=MagickFalse;
+          }
+          image_view=DestroyCacheView(image_view);
+        }
+      if ((bitmap->left+bitmap->bitmap.width) > metrics->width)
+        metrics->width=bitmap->left+bitmap->bitmap.width;
+      if ((draw_info->interword_spacing != 0.0) &&
+          (isspace((int) ((unsigned char) *p)) != 0) &&
+          (isspace((int) code) == 0))
+        origin.x+=64.0*draw_info->interword_spacing;
+      else
+        origin.x+=face->glyph->advance.x;
+      metrics->origin.x=origin.x;
+      metrics->origin.y=origin.y;
+      if (last_glyph.id != 0)
+        FT_Done_Glyph(last_glyph.image);
+      last_glyph=glyph;
+      code=(*p);
+    }
   if (last_glyph.id != 0)
     FT_Done_Glyph(last_glyph.image);
   if ((draw_info->stroke.opacity != TransparentOpacity) ||
@@ -1579,8 +1771,9 @@ static MagickBooleanType RenderPostscript(Image *image,
   else
     (void) fprintf(file,"/%s-ISO dup /%s ReencodeType findfont setfont\n",
       draw_info->font,draw_info->font);
-  (void) fprintf(file,"[%g %g %g %g 0 0] concat\n",draw_info->affine.sx,
-    -draw_info->affine.rx,-draw_info->affine.ry,draw_info->affine.sy);
+  (void) fprintf(file,"[%g %g %g %g 0 0] concat\n",
+    draw_info->affine.sx,-draw_info->affine.rx,-draw_info->affine.ry,
+    draw_info->affine.sy);
   text=EscapeParenthesis(draw_info->text);
   if (identity == MagickFalse)
     (void) fprintf(file,"(%s) stringwidth pop -0.5 mul -0.5 rmoveto\n",text);
@@ -1589,7 +1782,7 @@ static MagickBooleanType RenderPostscript(Image *image,
   (void) fprintf(file,"showpage\n");
   (void) fclose(file);
   (void) FormatMagickString(geometry,MaxTextExtent,"%ldx%ld+0+0!",(long)
-    (extent.x+0.5),(long) (extent.y+0.5));
+    floor(extent.x+0.5),(long) floor(extent.y+0.5));
   annotate_info=AcquireImageInfo();
   (void) FormatMagickString(annotate_info->filename,MaxTextExtent,"ps:%s",
     filename);
@@ -1629,7 +1822,8 @@ static MagickBooleanType RenderPostscript(Image *image,
       crop_info=GetImageBoundingBox(annotate_image,&annotate_image->exception);
       crop_info.height=(unsigned long) ((resolution.y/DefaultResolution)*
         ExpandAffine(&draw_info->affine)*draw_info->pointsize+0.5);
-      crop_info.y=(long) ((resolution.y/DefaultResolution)*extent.y/8.0+0.5);
+      crop_info.y=(long) ceil((resolution.y/DefaultResolution)*extent.y/8.0-
+        0.5);
       (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu%+ld%+ld",
         crop_info.width,crop_info.height,crop_info.x,crop_info.y);
       (void) TransformImage(&annotate_image,geometry,(char *) NULL);
@@ -1684,7 +1878,7 @@ static MagickBooleanType RenderPostscript(Image *image,
           x;
 
         register PixelPacket
-          *__restrict q;
+          *restrict q;
 
         q=GetCacheViewAuthenticPixels(annotate_view,0,y,annotate_image->columns,
           1,exception);
@@ -1693,7 +1887,7 @@ static MagickBooleanType RenderPostscript(Image *image,
         for (x=0; x < (long) annotate_image->columns; x++)
         {
           (void) GetFillColor(draw_info,x,y,&fill_color);
-          q->opacity=RoundToQuantum(QuantumRange-(((QuantumRange-
+          q->opacity=ClampToQuantum(QuantumRange-(((QuantumRange-
             (MagickRealType) PixelIntensityToQuantum(q))*(QuantumRange-
             fill_color.opacity))/QuantumRange));
           q->red=fill_color.red;
@@ -1707,8 +1901,8 @@ static MagickBooleanType RenderPostscript(Image *image,
       }
       annotate_view=DestroyCacheView(annotate_view);
       (void) CompositeImage(image,OverCompositeOp,annotate_image,
-        (long) (offset->x+0.5),(long) (offset->y-(metrics->ascent+
-        metrics->descent)+0.5));
+        (long) ceil(offset->x-0.5),(long) ceil(offset->y-(metrics->ascent+
+        metrics->descent)-0.5));
     }
   annotate_image=DestroyImage(annotate_image);
   return(MagickTrue);
@@ -1922,9 +2116,9 @@ static MagickBooleanType RenderX11(Image *image,const DrawInfo *draw_info,
           atan2(draw_info->affine.rx,draw_info->affine.sx);
     }
   (void) FormatMagickString(annotate_info.geometry,MaxTextExtent,
-    "%lux%lu+%ld+%ld",width,height,(long) (offset->x+0.5),
-    (long) (offset->y-metrics->ascent-metrics->descent+
-    draw_info->interline_spacing+0.5));
+    "%lux%lu+%ld+%ld",width,height,(long) ceil(offset->x-0.5),
+    (long) ceil(offset->y-metrics->ascent-metrics->descent+
+    draw_info->interline_spacing-0.5));
   pixel.pen_color.red=ScaleQuantumToShort(draw_info->fill.red);
   pixel.pen_color.green=ScaleQuantumToShort(draw_info->fill.green);
   pixel.pen_color.blue=ScaleQuantumToShort(draw_info->fill.blue);
