@@ -10,20 +10,20 @@
 %                         LLLLL  IIIII  SSSSS    T                            %
 %                                                                             %
 %                                                                             %
-%                       ImageMagick Image List Methods                        %
+%                        MagickCore Image List Methods                        %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                               December 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -41,9 +41,11 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/error.h"
-#include "magick/list.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
+#include "magick/list.h"
 #include "magick/memory_.h"
 #include "magick/string_.h"
 
@@ -58,7 +60,8 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  AppendImageToList() appends an image to the end of the list.
+%  AppendImageToList() appends the second image list to the end of the first
+%  list.  The given image list pointer is left unchanged, unless it was empty.
 %
 %  The format of the AppendImageToList method is:
 %
@@ -66,33 +69,33 @@
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
-%    o image: The image.
+%    o images: the image list to be appended to.
 %
+%    o image: the appended image or image list.
 %
 */
 MagickExport void AppendImageToList(Image **images,const Image *image)
 {
   register Image
-    *p;
+    *p,
+    *q;
 
   assert(images != (Image **) NULL);
   if (image == (Image *) NULL)
     return;
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((*images) == (Image *) NULL)
     {
       *images=(Image *) image;
       return;
     }
   assert((*images)->signature == MagickSignature);
-  for (p=(*images); p->next != (Image *) NULL; p=p->next);
-  p->next=(Image *) image;
-  p->next->previous=p;
-  SyncImageList(*images);
+  p=GetLastImageInList(*images);
+  q=GetFirstImageInList(image);
+  p->next=q;
+  q->previous=p;
 }
 
 /*
@@ -114,10 +117,9 @@ MagickExport void AppendImageToList(Image **images,const Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
-%    o exception: Return any errors or warnings in this structure.
+%    o images: the image list.
 %
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport Image *CloneImageList(const Image *images,ExceptionInfo *exception)
@@ -141,7 +143,7 @@ MagickExport Image *CloneImageList(const Image *images,ExceptionInfo *exception)
     if (clone == (Image *) NULL)
       {
         if (image != (Image *) NULL)
-          DestroyImageList(image);
+          image=DestroyImageList(image);
         return((Image *) NULL);
       }
     if (image == (Image *) NULL)
@@ -162,13 +164,120 @@ MagickExport Image *CloneImageList(const Image *images,ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   C l o n e I m a g e s                                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  CloneImages() clones one or more images from an image sequence, using a
+%  comma separated list of image numbers or ranges.
+%
+%  The numbers start at 0 for the first image in the list, while negative
+%  numbers refer to images starting counting from the end of the range. Images
+%  may be refered to multiple times to clone them multiple times. Images
+%  refered beyond the available number of images in list are ignored.
+%
+%  Images referenced may be reversed, and results in a clone of those images
+%  also being made with a reversed order.
+%
+%  The format of the CloneImages method is:
+%
+%      Image *CloneImages(const Image *images,const char *scenes,
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o images: the image sequence.
+%
+%    o scenes: This character string specifies which scenes to clone
+%      (e.g. 1,3-5,7-3,2).
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+MagickExport Image *CloneImages(const Image *images,const char *scenes,
+  ExceptionInfo *exception)
+{
+  char
+    *p;
+
+  const Image
+    *next;
+
+  Image
+    *clone_images,
+    *image;
+
+  long
+    first,
+    last,
+    step;
+
+  register long
+    i;
+
+  size_t
+    length;
+
+  assert(images != (const Image *) NULL);
+  assert(images->signature == MagickSignature);
+  assert(scenes != (char *) NULL);
+  if (images->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  clone_images=NewImageList();
+  images=GetFirstImageInList(images);
+  length=GetImageListLength(images);
+  for (p=(char *) scenes; *p != '\0';)
+  {
+    while ((isspace((int) ((unsigned char) *p)) != 0) || (*p == ','))
+      p++;
+    first=strtol(p,&p,10);
+    if (first < 0)
+      first+=(long) length;
+    last=first;
+    while (isspace((int) ((unsigned char) *p)) != 0)
+      p++;
+    if (*p == '-')
+      {
+        last=strtol(p+1,&p,10);
+        if (last < 0)
+          last+=(long) length;
+      }
+    for (step=first > last ? -1 : 1; first != (last+step); first+=step)
+    {
+      i=0;
+      for (next=images; next != (Image *) NULL; next=GetNextImageInList(next))
+      {
+        if (i == first)
+          {
+            image=CloneImage(next,0,0,MagickTrue,exception);
+            if (image == (Image *) NULL)
+              break;
+            AppendImageToList(&clone_images,image);
+          }
+        i++;
+      }
+    }
+  }
+  return(GetFirstImageInList(clone_images));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   D e l e t e I m a g e F r o m L i s t                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  DeleteImageFromList() deletes an image from the list.
+%  DeleteImageFromList() deletes an image from the list. List pointer
+%  is moved to the next image, if one is present. See RemoveImageFromList().
 %
 %  The format of the DeleteImageFromList method is:
 %
@@ -176,38 +285,137 @@ MagickExport Image *CloneImageList(const Image *images,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
+%    o images: the image list.
 %
 */
 MagickExport void DeleteImageFromList(Image **images)
 {
-  register Image
+  Image
+    *image;
+
+  image=RemoveImageFromList(images);
+  if (image != (Image *) NULL)
+    (void) DestroyImage(image);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   D e l e t e I m a g e s                                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  DeleteImages() deletes one or more images from an image sequence, using a
+%  comma separated list of image numbers or ranges.
+%
+%  The numbers start at 0 for the first image, while negative numbers refer to
+%  images starting counting from the end of the range. Images may be refered to
+%  multiple times without problems. Image refered beyond the available number
+%  of images in list are ignored.
+%
+%  If the referenced images are in the reverse order, that range will be
+%  completely ignored.  Unlike CloneImages().
+%
+%  The format of the DeleteImages method is:
+%
+%      DeleteImages(Image **images,const char *scenes,ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o images: the image sequence.
+%
+%    o scenes: This character string specifies which scenes to delete
+%      (e.g. 1,3-5,-2-6,2).
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+MagickExport void DeleteImages(Image **images,const char *scenes,
+  ExceptionInfo *exception)
+{
+  char
     *p;
 
+  Image
+    *image;
+
+  long
+    first,
+    last;
+
+  MagickBooleanType
+    *delete_list;
+
+  register long
+    i;
+
+  size_t
+    length;
+
   assert(images != (Image **) NULL);
-  if ((*images) == (Image *) NULL)
-    return;
   assert((*images)->signature == MagickSignature);
+  assert(scenes != (char *) NULL);
   if ((*images)->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*images)->filename);
-  p=(*images);
-  if ((p->previous == (Image *) NULL) && (p->next == (Image *) NULL))
-    *images=(Image *) NULL;
-  else
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      (*images)->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  *images=GetFirstImageInList(*images);
+  length=GetImageListLength(*images);
+  delete_list=(MagickBooleanType *) AcquireQuantumMemory(length,
+    sizeof(*delete_list));
+  if (delete_list == (MagickBooleanType *) NULL)
     {
-      if (p->previous != (Image *) NULL)
-        {
-          p->previous->next=p->next;
-          *images=p->previous;
-        }
-      if (p->next != (Image *) NULL)
-        {
-          p->next->previous=p->previous;
-          *images=p->next;
-        }
+      (void) ThrowMagickException(exception,GetMagickModule(),
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",(*images)->filename);
+      return;
     }
-  DestroyImage(p);
+  image=(*images);
+  for (i=0; i < (long) length; i++)
+    delete_list[i]=MagickFalse;
+  /*
+    Note which images will be deleted, avoid duplicate deleted
+  */
+  for (p=(char *) scenes; *p != '\0';)
+  {
+    while ((isspace((int)*p) != 0) || (*p == ','))
+      p++;
+    first=strtol(p,&p,10);
+    if (first < 0)
+      first+=(long) length;
+    last=first;
+    while (isspace((int) ((unsigned char) *p)) != 0)
+      p++;
+    if (*p == '-')
+      {
+        last=strtol(p+1,&p,10);
+        if (last < 0)
+          last+=(long) length;
+      }
+    if (first > last)
+      continue;
+    for (i=first; i <= last; i++)
+      if ((i >= 0) && (i < (long) length))
+        delete_list[i]=MagickTrue;
+  }
+  /*
+    Delete images marked for deletion, once only
+  */
+  image=(*images);
+  for (i=0; i < (long) length; i++)
+  {
+    *images=image;
+    image=GetNextImageInList(image);
+    if (delete_list[i] != MagickFalse)
+      DeleteImageFromList(images);
+
+  }
+  (void) RelinquishMagickMemory(delete_list);
+  *images=GetFirstImageInList(*images);
 }
 
 /*
@@ -225,33 +433,23 @@ MagickExport void DeleteImageFromList(Image **images)
 %
 %  The format of the DestroyImageList method is:
 %
-%      void DestroyImageList(Image *image)
+%      Image *DestroyImageList(Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image sequence.
-%
+%    o image: the image sequence.
 %
 */
-MagickExport void DestroyImageList(Image *images)
+MagickExport Image *DestroyImageList(Image *images)
 {
-  register Image
-    *p;
-
   if (images == (Image *) NULL)
-    return;
+    return((Image *) NULL);
   assert(images->signature == MagickSignature);
   if (images->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),images->filename);
-  for (p=images; p->previous != (Image *) NULL; p=p->previous);
-  for (images=p; p != (Image *) NULL; images=p)
-  {
-    p=p->next;
-    if (p != (Image *) NULL)
-      p->previous=(Image *) NULL;
-    images->next=(Image *) NULL;
-    DestroyImage(images);
-  }
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
+  while (images != (Image *) NULL)
+    DeleteImageFromList(&images);
+  return((Image *) NULL);
 }
 
 /*
@@ -273,7 +471,7 @@ MagickExport void DestroyImageList(Image *images)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
+%    o images: the image list.
 %
 */
 MagickExport Image *GetFirstImageInList(const Image *images)
@@ -284,7 +482,7 @@ MagickExport Image *GetFirstImageInList(const Image *images)
   if (images == (Image *) NULL)
     return((Image *) NULL);
   assert(images->signature == MagickSignature);
-  for (p=images; p->previous != (Image *) NULL; p=p->previous);
+  for (p=images; p->previous != (Image *) NULL; p=p->previous) ;
   return((Image *) p);
 }
 
@@ -307,28 +505,35 @@ MagickExport Image *GetFirstImageInList(const Image *images)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
-%    o index: The position within the list.
+%    o images: the image list.
 %
+%    o index: the position within the list.
 %
 */
 MagickExport Image *GetImageFromList(const Image *images,const long index)
 {
+  long
+    offset;
+
   register const Image
     *p;
 
   register long
     i;
 
+  size_t
+    length;
+
   if (images == (Image *) NULL)
     return((Image *) NULL);
   assert(images->signature == MagickSignature);
   if (images->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),images->filename);
-  for (p=images; p->previous != (Image *) NULL; p=p->previous);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
+  for (p=images; p->previous != (Image *) NULL; p=p->previous) ;
+  length=GetImageListLength(images);
+  for (offset=index; offset < 0; offset+=(long) length) ;
   for (i=0; p != (Image *) NULL; p=p->next)
-    if (i++ == ((index < 0) ? (long) GetImageListLength(images)+index : index))
+    if (i++ == (long) (offset % length))
       break;
   if (p == (Image *) NULL)
     return((Image *) NULL);
@@ -354,8 +559,7 @@ MagickExport Image *GetImageFromList(const Image *images,const long index)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
+%    o images: the image list.
 %
 */
 MagickExport long GetImageIndexInList(const Image *images)
@@ -391,8 +595,7 @@ MagickExport long GetImageIndexInList(const Image *images)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
+%    o images: the image list.
 %
 */
 MagickExport unsigned long GetImageListLength(const Image *images)
@@ -404,7 +607,7 @@ MagickExport unsigned long GetImageListLength(const Image *images)
     return(0);
   assert(images->signature == MagickSignature);
   if (images->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),images->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   while (images->previous != (Image *) NULL)
     images=images->previous;
   for (i=0; images != (Image *) NULL; images=images->next)
@@ -431,7 +634,7 @@ MagickExport unsigned long GetImageListLength(const Image *images)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
+%    o images: the image list.
 %
 */
 MagickExport Image *GetLastImageInList(const Image *images)
@@ -442,7 +645,7 @@ MagickExport Image *GetLastImageInList(const Image *images)
   if (images == (Image *) NULL)
     return((Image *) NULL);
   assert(images->signature == MagickSignature);
-  for (p=images; p->next != (Image *) NULL; p=p->next);
+  for (p=images; p->next != (Image *) NULL; p=p->next) ;
   return((Image *) p);
 }
 
@@ -465,8 +668,7 @@ MagickExport Image *GetLastImageInList(const Image *images)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
+%    o images: the image list.
 %
 */
 MagickExport Image *GetNextImageInList(const Image *images)
@@ -475,7 +677,7 @@ MagickExport Image *GetNextImageInList(const Image *images)
     return((Image *) NULL);
   assert(images->signature == MagickSignature);
   if (images->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),images->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   return(images->next);
 }
 
@@ -498,8 +700,7 @@ MagickExport Image *GetNextImageInList(const Image *images)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
+%    o images: the image list.
 %
 */
 MagickExport Image *GetPreviousImageInList(const Image *images)
@@ -524,8 +725,10 @@ MagickExport Image *GetPreviousImageInList(const Image *images)
 %  a sequential array.  For example,
 %
 %    group = ImageListToArray(images, exception);
-%    for (i = 0; i < n; i++)
-%      puts(group[i]->filename);
+%    while (i = 0; group[i] != (Image *) NULL; i++)
+%      printf("%s\n", group[i]->filename);
+%    printf("%d images\n", i);
+%    group = RelinquishMagickMemory(group);
 %
 %  The format of the ImageListToArray method is:
 %
@@ -533,10 +736,9 @@ MagickExport Image *GetPreviousImageInList(const Image *images)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image list.
-%
-%    o exception: Return any errors or warnings in this structure.
+%    o image: the image list.
 %
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport Image **ImageListToArray(const Image *images,
@@ -552,17 +754,16 @@ MagickExport Image **ImageListToArray(const Image *images,
     return((Image **) NULL);
   assert(images->signature == MagickSignature);
   if (images->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),images->filename);
-  group=(Image **) AcquireMagickMemory((size_t)
-    (GetImageListLength(images)+1)*sizeof(*group));
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
+  group=(Image **) AcquireQuantumMemory((size_t) GetImageListLength(images)+1UL,
+    sizeof(*group));
   if (group == (Image **) NULL)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed",images->filename);
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",images->filename);
       return((Image **) NULL);
     }
-  while (images->previous != (Image *) NULL)
-    images=images->previous;
+  images=GetFirstImageInList(images);
   for (i=0; images != (Image *) NULL; images=images->next)
     group[i++]=(Image *) images;
   group[i]=(Image *) NULL;
@@ -580,7 +781,9 @@ MagickExport Image **ImageListToArray(const Image *images,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  InsertImageInList() inserts an image in the list.
+%  InsertImageInList() inserts the second image or image list into the first
+%  image list immediatally after the image pointed to.  The given image list
+%  pointer is unchanged unless previously empty.
 %
 %  The format of the InsertImageInList method is:
 %
@@ -588,10 +791,9 @@ MagickExport Image **ImageListToArray(const Image *images,
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
-%    o image: The image.
+%    o images: the image list to insert into.
 %
+%    o image: the image list to insert.
 %
 */
 MagickExport void InsertImageInList(Image **images,Image *image)
@@ -603,13 +805,11 @@ MagickExport void InsertImageInList(Image **images,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((*images) == (Image *) NULL)
     return;
   assert((*images)->signature == MagickSignature);
   split=SplitImageList(*images);
-  if (split == (Image *) NULL)
-    return;
   AppendImageToList(images,image);
   AppendImageToList(images,split);
 }
@@ -656,10 +856,9 @@ MagickExport Image *NewImageList(void)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
-%    o image: The image.
+%    o images: the image list.
 %
+%    o image: the image.
 %
 */
 MagickExport void PrependImageToList(Image **images,Image *image)
@@ -678,7 +877,11 @@ MagickExport void PrependImageToList(Image **images,Image *image)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RemoveImageFromList() removes an image from the list.
+%  RemoveImageFromList() removes and returns the image pointed to.
+%
+%  The given image list pointer is set to point to the next image in list
+%  if it exists, otherwise it is set to the previous image, or NULL if list
+%  was emptied.
 %
 %  The format of the RemoveImageFromList method is:
 %
@@ -686,8 +889,7 @@ MagickExport void PrependImageToList(Image **images,Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
+%    o images: the image list.
 %
 */
 MagickExport Image *RemoveImageFromList(Image **images)
@@ -700,7 +902,8 @@ MagickExport Image *RemoveImageFromList(Image **images)
     return((Image *) NULL);
   assert((*images)->signature == MagickSignature);
   if ((*images)->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*images)->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      (*images)->filename);
   p=(*images);
   if ((p->previous == (Image *) NULL) && (p->next == (Image *) NULL))
     *images=(Image *) NULL;
@@ -716,6 +919,8 @@ MagickExport Image *RemoveImageFromList(Image **images)
           p->next->previous=p->previous;
           *images=p->next;
         }
+      p->previous=(Image *) NULL;
+      p->next=(Image *) NULL;
     }
   return(p);
 }
@@ -731,7 +936,11 @@ MagickExport Image *RemoveImageFromList(Image **images)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RemoveFirstImageFromList() removes the first image in the list.
+%  RemoveFirstImageFromList() removes and returns the first image in the list.
+%
+%  If the given image list pointer pointed to the removed first image, it is
+%  set to the new first image of list, or NULL if list was emptied, otherwise
+%  it is left as is.
 %
 %  The format of the RemoveFirstImageFromList method is:
 %
@@ -739,8 +948,7 @@ MagickExport Image *RemoveImageFromList(Image **images)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
+%    o images: the image list.
 %
 */
 MagickExport Image *RemoveFirstImageFromList(Image **images)
@@ -753,7 +961,8 @@ MagickExport Image *RemoveFirstImageFromList(Image **images)
     return((Image *) NULL);
   assert((*images)->signature == MagickSignature);
   if ((*images)->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*images)->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      (*images)->filename);
   image=(*images);
   while (image->previous != (Image *) NULL)
     image=image->previous;
@@ -778,7 +987,11 @@ MagickExport Image *RemoveFirstImageFromList(Image **images)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RemoveLastImageFromList() removes the last image from the list.
+%  RemoveLastImageFromList() removes and returns the last image from the list.
+%
+%  If the given image list pointer pointed to the removed last image, it is
+%  set to the new last image of list, or NULL if list was emptied, otherwise
+%  it is left as is.
 %
 %  The format of the RemoveLastImageFromList method is:
 %
@@ -786,8 +999,7 @@ MagickExport Image *RemoveFirstImageFromList(Image **images)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
+%    o images: the image list.
 %
 */
 MagickExport Image *RemoveLastImageFromList(Image **images)
@@ -800,7 +1012,8 @@ MagickExport Image *RemoveLastImageFromList(Image **images)
     return((Image *) NULL);
   assert((*images)->signature == MagickSignature);
   if ((*images)->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*images)->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      (*images)->filename);
   image=(*images);
   while (image->next != (Image *) NULL)
     image=image->next;
@@ -825,7 +1038,8 @@ MagickExport Image *RemoveLastImageFromList(Image **images)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ReplaceImageInList() replaces an image in the list.
+%  ReplaceImageInList() replaces an image in the list. Old image is destroyed.
+%  The given image list pointer is set to point to the just inserted image.
 %
 %  The format of the ReplaceImageInList method is:
 %
@@ -833,10 +1047,9 @@ MagickExport Image *RemoveLastImageFromList(Image **images)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
-%    o image: The image.
+%    o images: the image list.
 %
+%    o image: the image.
 %
 */
 MagickExport void ReplaceImageInList(Image **images,Image *image)
@@ -845,7 +1058,7 @@ MagickExport void ReplaceImageInList(Image **images,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((*images) == (Image *) NULL)
     return;
   assert((*images)->signature == MagickSignature);
@@ -855,7 +1068,7 @@ MagickExport void ReplaceImageInList(Image **images,Image *image)
   image->previous=(*images)->previous;
   if (image->previous != (Image *) NULL)
     image->previous->next=image;
-  DestroyImage(*images);
+  (void) DestroyImage(*images);
   (*images)=image;
 }
 
@@ -870,16 +1083,16 @@ MagickExport void ReplaceImageInList(Image **images,Image *image)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ReverseImageList() reverses the image list.
+%  ReverseImageList() reverses the order of an image list.
+%  The list pointer is reset to that start of the re-ordered list.
 %
 %  The format of the ReverseImageList method is:
 %
-%       ReverseImageList(const Image **images)
+%      void ReverseImageList(Image **images)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
+%    o images: the image list.
 %
 */
 MagickExport void ReverseImageList(Image **images)
@@ -895,8 +1108,9 @@ MagickExport void ReverseImageList(Image **images)
     return;
   assert((*images)->signature == MagickSignature);
   if ((*images)->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*images)->filename);
-  for (p=(*images); p->next != (Image *) NULL; p=p->next);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      (*images)->filename);
+  for (p=(*images); p->next != (Image *) NULL; p=p->next) ;
   *images=p;
   for ( ; p != (Image *) NULL; p=p->next)
   {
@@ -918,47 +1132,48 @@ MagickExport void ReverseImageList(Image **images)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  SpliceImageIntoList() removes 'length' images from the list and replaces
-%  them with the specified splice.
+%  them with the specified splice. Removed images are returned.
 %
 %  The format of the SpliceImageIntoList method is:
 %
-%      SpliceImageIntoList(Image **images,const unsigned unsigned long,
+%      SpliceImageIntoList(Image **images,const unsigned long,
 %        const Image *splice)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
+%    o images: the image list.
 %
-%    o length: The length of the image list to remove.
+%    o length: the length of the image list to remove.
 %
 %    o splice: Replace the removed image list with this list.
 %
-%
 */
-MagickExport void SpliceImageIntoList(Image **images,const unsigned long length,
-  const Image *splice)
+MagickExport Image *SpliceImageIntoList(Image **images,
+  const unsigned long length,const Image *splice)
 {
   Image
+    *image,
     *split;
 
-  register long
+  register unsigned long
     i;
 
   assert(images != (Image **) NULL);
   assert(splice != (Image *) NULL);
   assert(splice->signature == MagickSignature);
   if ((*images) == (Image *) NULL)
-    return;
+    return((Image *) NULL);
   assert((*images)->signature == MagickSignature);
   if ((*images)->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*images)->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      (*images)->filename);
   split=SplitImageList(*images);
-  if (split == (Image *) NULL)
-    return;
   AppendImageToList(images,splice);
-  for (i=0; (i < (long) length) && (split != (Image *) NULL); i++)
-    (void) RemoveImageFromList(&split);
+  image=(Image *) NULL;
+  for (i=0; (i < length) && (split != (Image *) NULL); i++)
+    AppendImageToList(&image,RemoveImageFromList(&split));
   AppendImageToList(images,split);
+  return(image);
 }
 
 /*
@@ -972,7 +1187,8 @@ MagickExport void SpliceImageIntoList(Image **images,const unsigned long length,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  SplitImageList() splits an image into two lists.
+%  SplitImageList() splits an image into two lists, after given image
+%  The list that was split off is returned, which may be empty.
 %
 %  The format of the SplitImageList method is:
 %
@@ -980,8 +1196,7 @@ MagickExport void SpliceImageIntoList(Image **images,const unsigned long length,
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
+%    o images: the image list.
 %
 */
 MagickExport Image *SplitImageList(Image *images)
@@ -1005,7 +1220,7 @@ MagickExport Image *SplitImageList(Image *images)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  SyncImageList() synchronizes the scenes in an image list.
+%  SyncImageList() synchronizes the scene numbers in an image list.
 %
 %  The format of the SyncImageList method is:
 %
@@ -1013,8 +1228,7 @@ MagickExport Image *SplitImageList(Image *images)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
+%    o images: the image list.
 %
 */
 MagickExport void SyncImageList(Image *images)
@@ -1060,8 +1274,7 @@ MagickExport void SyncImageList(Image *images)
 %
 %  A description of each parameter follows:
 %
-%    o images: The image list.
-%
+%    o images: the image list.
 %
 */
 MagickExport Image *SyncNextImageInList(const Image *images)
@@ -1076,6 +1289,7 @@ MagickExport Image *SyncNextImageInList(const Image *images)
       DestroyBlob(images->next);
       images->next->blob=ReferenceBlob(images->blob);
     }
+  images->next->compression=images->compression;
   images->next->endian=images->endian;
   return(images->next);
 }
