@@ -18,7 +18,7 @@
 %                                August 2007                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -65,12 +65,12 @@
 #define WLUT_WIDTH 1024
 struct _ResampleFilter
 {
+  CacheView
+    *view;
+
   Image
     *image;
 
-  ViewInfo
-    *view;
-
   ExceptionInfo
     *exception;
 
@@ -404,7 +404,7 @@ static MagickBooleanType InterpolateResampleFilter(
         alpha[i]=1.0;
         if (resample_filter->image->matte != MagickFalse)
           {
-            alpha[i]=QuantumScale*((MagickRealType) QuantumRange-p->opacity);
+            alpha[i]=QuantumScale*((MagickRealType) GetAlphaPixelComponent(p));
             pixels[i].red*=alpha[i];
             pixels[i].green*=alpha[i];
             pixels[i].blue*=alpha[i];
@@ -450,7 +450,7 @@ static MagickBooleanType InterpolateResampleFilter(
         alpha[i]=1.0;
         if (resample_filter->image->matte != MagickFalse)
           {
-            alpha[i]=QuantumScale*((MagickRealType) QuantumRange-p->opacity);
+            alpha[i]=QuantumScale*((MagickRealType) GetAlphaPixelComponent(p));
             pixels[i].red*=alpha[i];
             pixels[i].green*=alpha[i];
             pixels[i].blue*=alpha[i];
@@ -547,7 +547,7 @@ static MagickBooleanType InterpolateResampleFilter(
       RectangleInfo
         geometry;
 
-      ViewInfo
+      CacheView
         *filter_view;
 
       geometry.width=4L;
@@ -624,7 +624,7 @@ static MagickBooleanType InterpolateResampleFilter(
         alpha[i]=1.0;
         if (resample_filter->image->matte != MagickFalse)
           {
-            alpha[i]=QuantumScale*((MagickRealType) QuantumRange-p->opacity);
+            alpha[i]=QuantumScale*((MagickRealType) GetAlphaPixelComponent(p));
             pixels[i].red*=alpha[i];
             pixels[i].green*=alpha[i];
             pixels[i].blue*=alpha[i];
@@ -787,7 +787,7 @@ static MagickBooleanType InterpolateResampleFilter(
           alpha[n]=1.0;
           if (resample_filter->image->matte != MagickFalse)
             {
-              alpha[n]=QuantumScale*((MagickRealType) QuantumRange-p->opacity);
+              alpha[n]=QuantumScale*((MagickRealType) GetAlphaPixelComponent(p));
               pixels[n].red*=alpha[n];
               pixels[n].green*=alpha[n];
               pixels[n].blue*=alpha[n];
@@ -986,7 +986,7 @@ MagickExport MagickBooleanType ResamplePixelColor(
           Image
             *average_image;
 
-          ViewInfo
+          CacheView
             *average_view;
 
           GetMagickPixelPacket(resample_filter->image,
@@ -1132,13 +1132,13 @@ MagickExport MagickBooleanType ResamplePixelColor(
     Finialize results of resampling
   */
   divisor_m = 1.0/divisor_m;
-  pixel->opacity = (MagickRealType) RoundToQuantum(divisor_m*pixel->opacity);
+  pixel->opacity = (MagickRealType) ClampToQuantum(divisor_m*pixel->opacity);
   divisor_c = 1.0/divisor_c;
-  pixel->red   = (MagickRealType) RoundToQuantum(divisor_c*pixel->red);
-  pixel->green = (MagickRealType) RoundToQuantum(divisor_c*pixel->green);
-  pixel->blue  = (MagickRealType) RoundToQuantum(divisor_c*pixel->blue);
+  pixel->red   = (MagickRealType) ClampToQuantum(divisor_c*pixel->red);
+  pixel->green = (MagickRealType) ClampToQuantum(divisor_c*pixel->green);
+  pixel->blue  = (MagickRealType) ClampToQuantum(divisor_c*pixel->blue);
   if (resample_filter->image->colorspace == CMYKColorspace)
-    pixel->index = (MagickRealType) RoundToQuantum(divisor_c*pixel->index);
+    pixel->index = (MagickRealType) ClampToQuantum(divisor_c*pixel->index);
   return(MagickTrue);
 }
 
@@ -1155,19 +1155,30 @@ MagickExport MagickBooleanType ResamplePixelColor(
 %
 %  ScaleResampleFilter() does all the calculations needed to resample an image
 %  at a specific scale, defined by two scaling vectors.  This not using
-%  a orthogonal scaling, but two distorted scaling vectors.
+%  a orthogonal scaling, but two distorted scaling vectors, to allow the
+%  generation of a angled ellipse.
 %
-%  The input vectors are produced by either finding the derivitives of
-%  the distortion function, or the partial derivitives from a distortion
-%  mapping.  They do not need to be the orthogonal derivatives, but can be
-%  caclated using radial/angluar derivatives.
+%  As only two deritive scaling vectors are used the center of the ellipse
+%  must be the center of the lookup.  That is any curvature that the
+%  distortion may produce is discounted.
+%
+%  The input vectors are produced by either finding the derivitives of the
+%  distortion function, or the partial derivitives from a distortion mapping.
+%  They do not need to be the orthogonal dx,dy scaling vectors, but can be
+%  calculated from other derivatives.  For example you could use  dr,da/r
+%  polar coordinate vector scaling vectors
 %
 %  If   u,v =  DistortEquation(x,y)
-%  Then the scaling vectors are the function derivitives...
-%      du/dx, du/dy     and    dv/dx, dv/dy
+%  Then the scaling vectors dx,dy (in u,v space) are the derivitives...
+%      du/dx, dv/dx     and    du/dy, dv/dy
 %  If the scaling is only othogonally aligned then...
-%      dv/dx =  du/dy  =  0
-%  Producing a othogonally scaled circle for the area to be resampled.
+%      dv/dx = 0   and   du/dy  =  0
+%  Producing an othogonally alligned ellipse for the area to be resampled.
+%
+%  Note that scaling vectors are different to argument order.  Argument order
+%  is the general order the deritives are extracted from the distortion
+%  equations, EG: U(x,y), V(x,y).  Caution is advised if you are trying to
+%  define the ellipse directly from scaling vectors.
 %
 %  The format of the ScaleResampleFilter method is:
 %
@@ -1180,12 +1191,11 @@ MagickExport MagickBooleanType ResamplePixelColor(
 %      image being resampled
 %
 %    o dux,duy,dvx,dvy:
-%         The partial derivitives or scaling vectors for resampling
-%         for the X direction:  du/dx, dv/dx
-%         and the Y direction:  du/dy, dv/dy
-%         This defines the size and angle of the elliptical resampling area
-%         The order is the order you generally would figure out the
-%         deritives from the distortion equations.
+%         The partial derivitives or scaling vectors for resampling.
+%           dx = du/dx, dv/dx    and  dy = du/dy, dv/dy
+%
+%         The values are used to define the size and angle of the
+%         elliptical resampling area, centered on the lookup point.
 %
 */
 MagickExport void ScaleResampleFilter(ResampleFilter *resample_filter,
@@ -1208,6 +1218,8 @@ MagickExport void ScaleResampleFilter(ResampleFilter *resample_filter,
   /* Find Ellipse Coefficents such that
         A*u^2 + B*u*v + C*v^2 = F
      With u,v relative to point around which we are resampling.
+     And the given scaling dx,dy vectors in u,v space
+         du/dx,dv/dx   and  du/dy,dv/dy
   */
 #if 0
   /* Direct conversions of derivatives to elliptical coefficients
@@ -1225,7 +1237,7 @@ MagickExport void ScaleResampleFilter(ResampleFilter *resample_filter,
      to do both Reconstruction and Prefiltering of the pixels in the
      resampling.  It also means it is likely to have at least 4 pixels within
      the area of the ellipse, for weighted averaging.
-     No scaling will result in F == 4.0 and a circle of radius 2.0
+     No scaling will result if F == 4.0 and a circle of radius 2.0
   */
   A = dvx*dvx+dvy*dvy+1;
   B = (dux*dvx+duy*dvy)*-2.0;
@@ -1233,8 +1245,53 @@ MagickExport void ScaleResampleFilter(ResampleFilter *resample_filter,
   F = A*C - B*B/4;
 #define F_UNITY 4.0
 #endif
+
+/* DEBUGGING OUTPUT */
 #if 0
-   printf ("F = %lf\n", F);
+  fprintf(stderr, "dux=%lf; dvx=%lf;   duy=%lf; dvy%lf;\n",
+       dux, dvx, duy, dvy);
+  fprintf(stderr, "A=%lf; B=%lf; C=%lf; F=%lf\n", A,B,C,F);
+#endif
+
+#if 0
+  /* Figure out the Ellipses Major and Minor Axis, and other info.
+     This information currently not needed at this time, but may be
+     needed later for better limit determination.
+  */
+  { double alpha, beta, gamma, Major, Minor;
+    double Eccentricity, Ellipse_Area, Ellipse_angle;
+    double max_horizontal_cross_section, max_vertical_cross_section;
+    alpha = A+C;
+    beta  = A-C;
+    gamma = sqrt(beta*beta + B*B );
+
+    if ( alpha - gamma <= MagickEpsilon )
+      Major = MagickHuge;
+    else
+      Major = sqrt(2*F/(alpha - gamma));
+    Minor = sqrt(2*F/(alpha + gamma));
+
+    fprintf(stderr, "\tMajor=%lf; Minor=%lf\n",
+         Major, Minor );
+
+    /* other information about ellipse include... */
+    Eccentricity = Major/Minor;
+    Ellipse_Area = MagickPI*Major*Minor;
+    Ellipse_angle =  atan2(B, A-C);
+
+    fprintf(stderr, "\tAngle=%lf Area=%lf\n",
+         RadiansToDegrees(Ellipse_angle), Ellipse_Area );
+
+    /* Ellipse limits */
+
+    /* orthogonal rectangle - improved ellipse */
+    max_horizontal_orthogonal = sqrt(A); /* = sqrt(4*A*F/(4*A*C-B*B)) */
+    max_vertical_orthogonal   = sqrt(C); /* = sqrt(4*C*F/(4*A*C-B*B)) */
+
+    /* parallelogram bounds -- what we are using */
+    max_horizontal_cross_section = sqrt(F/A);
+    max_vertical_cross_section   = sqrt(F/C);
+  }
 #endif
 
   /* Is default elliptical area, too small? Image being magnified?
@@ -1255,34 +1312,6 @@ MagickExport void ScaleResampleFilter(ResampleFilter *resample_filter,
     return;
   }
 
-#if 0
-  /* Figure out the Ellipses Major and Minor Axis, and other info.
-     This information currently not needed at this time, but may be
-     needed later for better limit determination.
-  */
-  { double alpha, beta, gamma, Major, Minor;
-    alpha = A+C;
-    beta  = A-C;
-    gamma = sqrt(beta*beta + B*B );
-
-    if ( alpha - gamma <= MagickEpsilon )
-      Major = MagickHuge;
-    else
-      Major = sqrt(2*F/(alpha - gamma));
-    Minor = sqrt(2*F/(alpha + gamma));
-
-    /* other information about ellipse include...
-    Eccentricity = Major/Minor;
-    Ellipse_Area = MagickPI*Major*Minor
-    ellipse_angle =  arctan2(B, A-C);
-
-    This actually used as part of parallelogram bounds
-    max_horizontal_cross_section =  2*sqrt(F/A) = sqrt(C-B*B/A);
-    max_vertical_cross_section   =  2*sqrt(F/C) = sqrt(A-B*B/C);
-    */
-  }
-#endif
-
   /* Othogonal bounds of the ellipse */
   resample_filter->sqrtA = sqrt(A)+1.0;     /* Vertical Orthogonal Limit */
   resample_filter->sqrtC = sqrt(C)+1.0;     /* Horizontal Orthogonal Limit */
@@ -1515,6 +1544,7 @@ MagickExport MagickBooleanType SetResampleFilterVirtualPixelMethod(
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       resample_filter->image->filename);
   resample_filter->virtual_pixel=method;
-  (void) SetCacheViewVirtualPixelMethod(resample_filter->view,method);
+  if (method != UndefinedVirtualPixelMethod)
+    (void) SetCacheViewVirtualPixelMethod(resample_filter->view,method);
   return(MagickTrue);
 }
